@@ -23,10 +23,10 @@ import os.path as osp
 
 # Local imports
 from spyderlib.baseconfig import _
-from spyderlib.config import get_icon, CONF
+from spyderlib.config import CONF
 from spyderlib.utils.misc import get_error_match
-from spyderlib.utils.qthelpers import (create_action, add_actions, mimedata2url,
-                                       DialogManager)
+from spyderlib.utils.qthelpers import (get_icon, create_action, add_actions,
+                                       mimedata2url, DialogManager)
 from spyderlib.utils.environ import EnvDialog
 from spyderlib.widgets.internalshell import InternalShell
 from spyderlib.widgets.findreplace import FindReplace
@@ -40,8 +40,7 @@ class Console(SpyderPluginWidget):
     """
     CONF_SECTION = 'internal_console'
     def __init__(self, parent=None, namespace=None, commands=[], message="",
-                 debug=False, exitfunc=None, profile=False,
-                 multithreaded=False):
+                 exitfunc=None, profile=False, multithreaded=False):
         SpyderPluginWidget.__init__(self, parent)
         
         self.dialog_manager = DialogManager()
@@ -50,8 +49,8 @@ class Console(SpyderPluginWidget):
         light_background = self.get_option('light_background')
         self.shell = InternalShell(parent, namespace, commands, message,
                                    self.get_option('max_line_count'),
-                                   self.get_plugin_font(),
-                                   debug, exitfunc, profile, multithreaded,
+                                   self.get_plugin_font(), exitfunc, profile,
+                                   multithreaded,
                                    light_background=light_background)
         self.connect(self.shell, SIGNAL('status(QString)'),
                      lambda msg:
