@@ -15,9 +15,10 @@ import os.path as osp
 # Local imports
 from spyderlib.utils import encoding
 from spyderlib.baseconfig import _
-from spyderlib.config import get_icon, CONF, get_color_scheme
-from spyderlib.utils.qthelpers import (create_action, create_toolbutton,
-                                       add_actions)
+from spyderlib.config import CONF
+from spyderlib.guiconfig import get_color_scheme
+from spyderlib.utils.qthelpers import (get_icon, create_action,
+                                       create_toolbutton, add_actions)
 from spyderlib.widgets.tabs import Tabs
 from spyderlib.widgets.sourcecode import codeeditor
 from spyderlib.widgets.findreplace import FindReplace
@@ -201,7 +202,7 @@ class HistoryLog(SpyderPluginWidget):
         Add new history tab
         Slot for SIGNAL('add_history(QString)') emitted by shell instance
         """
-        filename = encoding.to_unicode(filename)
+        filename = encoding.to_unicode_from_fs(filename)
         if filename in self.filenames:
             return
         editor = codeeditor.CodeEditor(self)
@@ -239,7 +240,9 @@ class HistoryLog(SpyderPluginWidget):
         Slot for SIGNAL('append_to_history(QString,QString)')
         emitted by shell instance
         """
-        filename, command = encoding.to_unicode(filename), unicode(command)
+        if not isinstance(filename, basestring): # filename is a QString
+            filename = unicode(filename.toUtf8(), 'utf-8')
+        command = unicode(command)
         index = self.filenames.index(filename)
         self.editors[index].append(command)
         if self.get_option('go_to_eof'):
