@@ -10,40 +10,41 @@ from spyderlib.qt.QtGui import (QHBoxLayout, QVBoxLayout, QLabel, QSizePolicy,
                                 QMenu, QToolButton, QGroupBox, QFontComboBox,
                                 QActionGroup, QFontDialog, QWidget, QComboBox,
                                 QLineEdit, QMessageBox)
-from spyderlib.qt.QtCore import SIGNAL, QUrl, QTimer
+from spyderlib.qt.QtCore import SIGNAL, QUrl, QTimer, QThread
 
 import re
 import os.path as osp
 import socket
+import sys
 
 # Local imports
-from spyderlib.baseconfig import get_conf_path, _
-from spyderlib.config import (get_icon, CONF, get_color_scheme,
-                              get_font, set_font)
+from spyderlib.baseconfig import get_conf_path, _, SUPPORTED_IPYTHON
+from spyderlib.config import CONF
+from spyderlib.guiconfig import get_color_scheme, get_font, set_font
 from spyderlib.utils import programs
-from spyderlib.utils.qthelpers import (create_toolbutton, add_actions,
-                                       create_action)
+from spyderlib.utils.qthelpers import (get_icon, create_toolbutton,
+                                       add_actions, create_action)
 from spyderlib.widgets.comboboxes import EditableComboBox
 from spyderlib.widgets.sourcecode import codeeditor
 from spyderlib.widgets.findreplace import FindReplace
 from spyderlib.widgets.browser import WebView
 from spyderlib.widgets.externalshell.pythonshell import ExtPythonShellWidget
+
+#XXX: hardcoded dependency on optional IPython plugin component
+#     that requires the hack to make this work without IPython
+if programs.is_module_installed('IPython.frontend.qt', SUPPORTED_IPYTHON):
+    from spyderlib.widgets.ipython import IPythonControlWidget
+else:
+    IPythonControlWidget = None  # analysis:ignore
+
 from spyderlib.plugins import SpyderPluginWidget, PluginConfigPage
 
 try:
-    from spyderlib.utils.sphinxify import CSS_PATH, sphinxify
-    HTML_HEAD = '<html> \
-    <head> \
-    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" /> \
-    <link rel="stylesheet" href="%s/default.css" type="text/css" /> \
-    <link rel="stylesheet" href="%s/pygments.css" type="text/css" /> \
-    </head> \
-    <body>' % (CSS_PATH, CSS_PATH)
-    
-    HTML_TAIL = '</body> \
-    </html>'
+    from sphinx import __version__ as sphinx_version
+    from spyderlib.utils.inspector.sphinxify import (CSS_PATH, sphinxify,
+                                                     warning, generate_context)
 except ImportError:
-    sphinxify = None
+    sphinxify = sphinx_version = None  # analysis:ignore
 
 
 class ObjectComboBox(EditableComboBox):
@@ -62,23 +63,22 @@ class ObjectComboBox(EditableComboBox):
             qstr = self.currentText()
         if not re.search('^[a-zA-Z0-9_\.]*$', str(qstr), 0):
             return False
-        shell = self.object_inspector.shell
         objtxt = unicode(qstr)
-        if shell is not None:
-            self.object_inspector._check_if_shell_is_running()
-            if self.object_inspector.get_option('automatic_import'):
-                shell = self.object_inspector.internal_shell
+        if self.object_inspector.get_option('automatic_import'):
+            shell = self.object_inspector.internal_shell
+            if shell is not None:
                 return shell.is_defined(objtxt, force_import=True)
-            else:
+        shell = self.object_inspector.get_shell()
+        if shell is not None:
+            try:
+                return shell.is_defined(objtxt)
+            except socket.error:
+                shell = self.object_inspector.get_shell()
                 try:
                     return shell.is_defined(objtxt)
                 except socket.error:
-                    self.object_inspector._check_if_shell_is_running()
-                    try:
-                        return shell.is_defined(objtxt)
-                    except socket.error:
-                        # Well... too bad!
-                        pass
+                    # Well... too bad!
+                    pass
         
     def validate_current_text(self):
         self.validate(self.currentText())
@@ -87,11 +87,6 @@ class ObjectInspectorConfigPage(PluginConfigPage):
     def setup_page(self):
         sourcecode_group = QGroupBox(_("Source code"))
         wrap_mode_box = self.create_checkbox(_("Wrap lines"), 'wrap')
-        plain_text_font_group = self.create_fontgroup(option=None,
-                                    text=_("Plain text font style"),
-                                    fontfilters=QFontComboBox.MonospacedFonts)
-        rich_text_font_group = self.create_fontgroup(option='rich_text',
-                                text=_("Rich text font style"))
         names = CONF.get('color_schemes', 'names')
         choices = zip(names, names)
         cs_combo = self.create_combobox(_("Syntax color scheme: "),
@@ -102,9 +97,32 @@ class ObjectInspectorConfigPage(PluginConfigPage):
         sourcecode_layout.addWidget(cs_combo)
         sourcecode_group.setLayout(sourcecode_layout)
         
+        plain_text_font_group = self.create_fontgroup(option=None,
+                                    text=_("Plain text font style"),
+                                    fontfilters=QFontComboBox.MonospacedFonts)
+        rich_text_font_group = self.create_fontgroup(option='rich_text',
+                                text=_("Rich text font style"))
+                                
+        features_group = QGroupBox(_("Additional features"))
+        math_box = self.create_checkbox(_("Render mathematical equations"),
+                                        'math')
+        req_sphinx = sphinx_version is not None and sphinx_version >= "1.1"
+        math_box.setEnabled(req_sphinx)
+        if not req_sphinx:
+            sphinx_tip = _("This feature requires Sphinx 1.1 or superior.")
+            if sphinx_version is not None:
+                sphinx_tip += "\n" + _("Sphinx %s is currently installed."
+                                       ) % sphinx_version
+            math_box.setToolTip(sphinx_tip)
+        
+        features_layout = QVBoxLayout()
+        features_layout.addWidget(math_box)
+        features_group.setLayout(features_layout)
+        
         vlayout = QVBoxLayout()
         vlayout.addWidget(rich_text_font_group)
         vlayout.addWidget(plain_text_font_group)
+        vlayout.addWidget(features_group)
         vlayout.addWidget(sourcecode_group)
         vlayout.addStretch(1)
         self.setLayout(vlayout)
@@ -189,6 +207,42 @@ class PlainText(QWidget):
         self.editor.clear()
 
 
+class SphinxThread(QThread):
+    """
+    A worker thread for handling rich text rendering.
+    
+    Parameters
+    ----------
+    text : dict
+        A dict containing the doc string components to be rendered.
+    html_text_no_doc : unicode
+        Text to be rendered if doc string cannot be extracted.
+    math_option : bool
+        Use LaTeX math rendering.
+        
+    """
+    def __init__(self, text={}, html_text_no_doc='', math_option=False):
+        super(SphinxThread, self).__init__()
+        self.text = text
+        self.html_text_no_doc = html_text_no_doc
+        self.math_option = math_option
+
+    def run(self):
+        html_text = self.html_text_no_doc
+        text = self.text
+        if text is not None and text['docstring'] != '':
+            try:
+                context = generate_context(name=text['name'],
+                                           argspec=text['argspec'],
+                                           note=text['note'],
+                                           math=self.math_option)
+                html_text = sphinxify(text['docstring'], context)
+            except Exception, error:
+                self.emit(SIGNAL('error_msg(QString)'), unicode(error))
+                return
+        self.emit(SIGNAL('html_ready(QString)'), html_text)
+
+
 class ObjectInspector(SpyderPluginWidget):
     """
     Docstrings viewer widget
@@ -198,6 +252,8 @@ class ObjectInspector(SpyderPluginWidget):
     LOG_PATH = get_conf_path('.inspector')
     def __init__(self, parent):
         SpyderPluginWidget.__init__(self, parent)
+        
+        self.internal_shell = None
 
         # Initialize plugin
         self.initialize_plugin()
@@ -241,7 +297,11 @@ class ObjectInspector(SpyderPluginWidget):
         # Object name
         layout_edit = QHBoxLayout()
         layout_edit.setContentsMargins(0, 0, 0, 0)
-        source_label = QLabel(_("Source"))
+        txt = _("Source")
+        if sys.platform == 'darwin':
+            source_label = QLabel("  " + txt)
+        else:
+            source_label = QLabel(txt)
         layout_edit.addWidget(source_label)
         self.source_combo = QComboBox(self)
         self.source_combo.addItems([_("Console"), _("Editor")])
@@ -325,6 +385,18 @@ class ObjectInspector(SpyderPluginWidget):
         layout.addWidget(self.rich_text)
         self.setLayout(layout)
         
+        # Add worker thread for handling rich text rendering
+        if sphinxify is None:
+            self._sphinx_thread = None
+        else:
+            self._sphinx_thread = SphinxThread(text={},
+                                  html_text_no_doc=warning(self.no_doc_string),
+                                  math_option=self.get_option('math'))
+            self.connect(self._sphinx_thread, SIGNAL('html_ready(QString)'), 
+                         self._on_sphinx_thread_html_ready)
+            self.connect(self._sphinx_thread, SIGNAL('error_msg(QString)'),
+                         self._on_sphinx_thread_error_msg)
+
         self._starting_up = True
             
     #------ SpyderPluginWidget API ---------------------------------------------    
@@ -378,6 +450,8 @@ class ObjectInspector(SpyderPluginWidget):
         wrap_n = 'wrap'
         wrap_o = self.get_option(wrap_n)
         self.wrap_action.setChecked(wrap_o)
+        math_n = 'math'
+        math_o = self.get_option(math_n)
         if font_n in options:
             scs = color_scheme_o if color_scheme_n in options else None
             self.set_plain_text_font(font_o, color_scheme=scs)
@@ -387,6 +461,8 @@ class ObjectInspector(SpyderPluginWidget):
             self.set_plain_text_color_scheme(color_scheme_o)
         if wrap_n in options:
             self.toggle_wrap_mode(wrap_o)
+        if math_n in options:
+            self.toggle_math_mode(math_o)
         
     #------ Public API (related to inspector's source) -------------------------
     def source_is_console(self):
@@ -471,7 +547,11 @@ class ObjectInspector(SpyderPluginWidget):
         """Toggle wrap mode"""
         self.plain_text.editor.toggle_wrap_mode(checked)
         self.set_option('wrap', checked)
-        
+    
+    def toggle_math_mode(self, checked):
+        """Toggle math mode"""
+        self.set_option('math', checked)
+    
     def is_plain_text_mode(self):
         """Return True if plain text mode is active"""
         return self.plain_text.isVisible()
@@ -496,9 +576,35 @@ class ObjectInspector(SpyderPluginWidget):
         self.show_source_action.setChecked(False)
             
     def set_plain_text(self, text, is_code):
-        """Set plain text"""
-        self.plain_text.set_text(text, is_code)
-        self.save_text([self.plain_text.set_text, text, is_code])
+        """Set plain text docs"""
+        
+        # text is coming from utils.dochelpers.getdoc
+        if type(text) is dict:
+            name = text['name']
+            if name:
+                rst_title = ''.join(['='*len(name), '\n', name, '\n',
+                                    '='*len(name), '\n\n'])
+            else:
+                rst_title = ''
+            
+            if text['argspec']:
+                definition = ''.join(['Definition: ', name, text['argspec'],
+                                      '\n'])
+            else:
+                definition = ''
+            
+            if text['note']:
+                note = ''.join(['Type: ', text['note'], '\n\n----\n\n'])
+            else:
+                note = ''
+
+            full_text = ''.join([rst_title, definition, note,
+                                 text['docstring']])
+        else:
+            full_text = text
+        
+        self.plain_text.set_text(full_text, is_code)
+        self.save_text([self.plain_text.set_text, full_text, is_code])
         
     def set_rich_text_html(self, html_text, base_url):
         """Set rich text"""
@@ -513,8 +619,8 @@ class ObjectInspector(SpyderPluginWidget):
         if self.source_is_console():
             self.set_object_text(None, force_refresh=True)
         elif self._last_rope_data is not None:
-            objtxt, doctxt = self._last_rope_data
-            self.set_rope_doc(objtxt, doctxt, force_refresh=True)
+            text = self._last_rope_data
+            self.set_rope_doc(text, force_refresh=True)
     
     def set_object_text(self, text, force_refresh=False, ignore_unknown=False):
         """Set object analyzed by Object Inspector"""
@@ -542,24 +648,28 @@ class ObjectInspector(SpyderPluginWidget):
         if self.dockwidget is not None:
             self.dockwidget.blockSignals(False)
         
-    def set_rope_doc(self, objtxt, doctxt, force_refresh=False):
-        """Use the object inspector to show text computed with rope
-        from the editor plugin"""
+    def set_rope_doc(self, text, force_refresh=False):
+        """
+        Use the object inspector to show text computed with rope from the
+        Editor plugin
+        """
         if (self.locked and not force_refresh):
             return
         self.switch_to_editor_source()
         
-        self._last_rope_data = objtxt, doctxt
+        self._last_rope_data = text
+        
+        self.object_edit.setText(text['obj_text'])
         
-        self.object_edit.setText(objtxt)
         if self.rich_help:
-            self.set_sphinx_text(doctxt)
+            self.set_sphinx_text(text)
         else:
-            self.set_plain_text(doctxt, is_code=False)
+            self.set_plain_text(text, is_code=False)
         
         if self.dockwidget is not None:
             self.dockwidget.blockSignals(True)
-        self.__eventually_raise_inspector(doctxt, force=force_refresh)
+        self.__eventually_raise_inspector(text['docstring'],
+                                          force=force_refresh)
         if self.dockwidget is not None:
             self.dockwidget.blockSignals(False)
             
@@ -619,9 +729,9 @@ class ObjectInspector(SpyderPluginWidget):
         
     def toggle_auto_import(self, checked):
         """Toggle automatic import feature"""
-        self.force_refresh()
         self.combo.validate_current_text()
         self.set_option('automatic_import', checked)
+        self.force_refresh()
         
     def toggle_locked(self):
         """
@@ -640,72 +750,67 @@ class ObjectInspector(SpyderPluginWidget):
         
     def set_shell(self, shell):
         """Bind to shell"""
-        self.shell = shell
-
-    def get_running_python_shell(self):
-        shell = None
-        if self.external_console is not None:
-            shell = self.external_console.get_running_python_shell()
-        if shell is None:
-            shell = self.internal_shell
-        return shell
-        
-    def shell_terminated(self, shell):
-        """
-        External shall has terminated:
-        binding object inspector to another shell
-        """
-        if self.shell is shell:
-            self.shell = self.get_running_python_shell()
-        
-    def _check_if_shell_is_running(self):
-        """
-        Checks if bound external shell is still running.
-        Otherwise, switch to internal console
-        """
+        if IPythonControlWidget is not None:
+            # XXX(anatoli): hack to make Spyder run on systems without IPython
+            #               there should be a better way
+            if isinstance(shell, IPythonControlWidget):
+                # XXX: this ignores passed argument completely
+                self.shell = self.external_console.get_current_shell()
+        else:
+            self.shell = shell
+    
+    def get_shell(self):
+        """Return shell which is currently bound to object inspector,
+        or another running shell if it has been terminated"""
         if not isinstance(self.shell, ExtPythonShellWidget) \
            or not self.shell.externalshell.is_running():
-            self.shell = self.get_running_python_shell()
+            self.shell = None
+            if self.external_console is not None:
+                self.shell = self.external_console.get_running_python_shell()
+            if self.shell is None:
+                self.shell = self.internal_shell
+        return self.shell
         
     def set_sphinx_text(self, text):
         """Sphinxify text and display it"""
-        if text is not None:
-            try:
-                html_text = sphinxify(text)
-            except Exception, error:
-                import sphinx
-                QMessageBox.critical(self,
-                            _('Object inspector'),
-                            _("The following error occured when calling "
-                              "<b>Sphinx %s</b>. <br>Please check if this "
-                              "version of Sphinx is supported by Spyder."
-                              "<br><br>Error message:<br>%s"
-                              ) % (sphinx.__version__, str(error)))
-                self.plain_text_action.setChecked(True)
-                return
-        else:
-            html_text = '<div id=\"warning\">'+self.no_doc_string+'</div>'
-        html_text = HTML_HEAD + html_text + HTML_TAIL
-        self.set_rich_text_html(html_text, QUrl.fromLocalFile(CSS_PATH))
+        # If the thread is already running wait for it to finish before
+        # starting it again.
+        if self._sphinx_thread.wait():
+            # Math rendering option could have changed
+            self._sphinx_thread.math_option = self.get_option('math')
+            self._sphinx_thread.text = text
+            self._sphinx_thread.start()
         
+    def _on_sphinx_thread_html_ready(self, html_text):
+        """Set our sphinx documentation based on thread result"""
+        self.set_rich_text_html(html_text, QUrl.fromLocalFile(CSS_PATH))
+
+    def _on_sphinx_thread_error_msg(self, error_msg):
+        """ Display error message on Sphinx rich text failure"""
+        self.plain_text_action.setChecked(True)
+        QMessageBox.critical(self,
+                    _('Object inspector'),
+                    _("The following error occured when calling "
+                      "<b>Sphinx %s</b>. <br>Incompatible Sphinx "
+                      "version or doc string decoding failed."
+                      "<br><br>Error message:<br>%s"
+                      ) % (sphinx_version, error_msg))
+
     def show_help(self, obj_text, ignore_unknown=False):
         """Show help"""
-        if self.shell is None:
-            return
-        self._check_if_shell_is_running()
-        if self.shell is None:
+        shell = self.get_shell()
+        if shell is None:
             return
         obj_text = unicode(obj_text)
         
-        if self.shell.is_defined(obj_text):
-            shell = self.shell
-        elif self.get_option('automatic_import') and \
-             self.internal_shell.is_defined(obj_text, force_import=True):
-            shell = self.internal_shell
-        else:
-            shell = None
-            doc_text = None
-            source_text = None
+        if not shell.is_defined(obj_text):
+            if self.get_option('automatic_import') and\
+               self.internal_shell.is_defined(obj_text, force_import=True):
+                shell = self.internal_shell
+            else:
+                shell = None
+                doc_text = None
+                source_text = None
             
         if shell is not None:
             doc_text = shell.get_doc(obj_text)
