@@ -378,9 +378,10 @@ class MainWindow(QMainWindow):
         self.debug_toolbar_actions = []
         
         # Set Window title and icon
-        title = "Spyder"
+        title = "Spyder (Python %s.%s)" % (sys.version_info.major,
+                                           sys.version_info.minor)
         if DEBUG:
-            title += " (DEBUG MODE)"
+            title += " [DEBUG MODE]"
         self.setWindowTitle(title)
         icon_name = 'spyder_light.svg' if self.light else 'spyder.svg'
         # Resampling SVG icon only on non-Windows platforms (see Issue 1314):
@@ -1029,7 +1030,10 @@ class MainWindow(QMainWindow):
         """Actions to be performed only after the main window's `show` method 
         was triggered"""
         self.emit(SIGNAL('restore_scrollbar_position()'))
-        self.extconsole.open_interpreter_at_startup()
+        if self.light:
+            self.extconsole.open_interpreter()
+        else:
+            self.extconsole.open_interpreter_at_startup()
         self.extconsole.setMinimumHeight(0)
         if self.projectexplorer is not None:
             self.projectexplorer.check_for_io_errors()
