@@ -6,14 +6,21 @@
 
 """Running programs utilities"""
 
+from __future__ import with_statement
+
+from distutils.version import LooseVersion
+import imp
+import inspect
 import os
 import os.path as osp
-import sys
-import subprocess
-import imp
 import re
+import subprocess
+import sys
+import tempfile
+
+
+TEMPDIR = tempfile.gettempdir() + osp.sep + 'spyder'
 
-from spyderlib.utils import encoding
 
 def is_program_installed(basename):
     """Return program absolute path if installed in PATH
@@ -55,6 +62,11 @@ def start_file(filename):
     Returns True if successfull, otherwise returns False."""
     from spyderlib.qt.QtGui import QDesktopServices
     from spyderlib.qt.QtCore import QUrl
+
+    # We need to use setUrl instead of setPath because this is the only
+    # cross-platform way to open external files. setPath fails completely on
+    # Mac and doesn't open non-ascii files on Linux.
+    # Fixes Issue 740
     url = QUrl()
     url.setUrl(filename)
     return QDesktopServices.openUrl(url)
@@ -137,13 +149,13 @@ def run_python_script_in_terminal(fname, wdir, args, interact,
     p_args += get_python_args(fname, python_args, interact, debug, args)
     
     if os.name == 'nt':
+        cmd = 'start cmd.exe /c "cd %s && ' % wdir + ' '.join(p_args) + '"'
         # Command line and cwd have to be converted to the filesystem
         # encoding before passing them to subprocess
         # See http://bugs.python.org/issue1759845#msg74142
-        cmd = encoding.to_fs_from_unicode(
-                'start cmd.exe /c "cd %s && ' % wdir + ' '.join(p_args) + '"')
-        subprocess.Popen(cmd, shell=True,
-                         cwd=encoding.to_fs_from_unicode(wdir))
+        from spyderlib.utils.encoding import to_fs_from_unicode
+        subprocess.Popen(to_fs_from_unicode(cmd), shell=True,
+                         cwd=to_fs_from_unicode(wdir))
     elif os.name == 'posix':
         cmd = 'gnome-terminal'
         if is_program_installed(cmd):
@@ -161,60 +173,114 @@ def run_python_script_in_terminal(fname, wdir, args, interact,
         raise NotImplementedError
 
 
-def is_module_installed(module_name, version=None):
+def check_version(actver, version, cmp_op):
+    """
+    Check version string of an active module against a required version.
+
+    If dev/prerelease tags result in TypeError for string-number comparison,
+    it is assumed that the dependency is satisfied.
+    Users on dev branches are responsible for keeping their own packages up to
+    date.
+    
+    Copyright (C) 2013  The IPython Development Team
+
+    Distributed under the terms of the BSD License.
+    """
+    try:
+        if cmp_op == '>':
+            return LooseVersion(actver) > LooseVersion(version)
+        elif cmp_op == '>=':
+            return LooseVersion(actver) >= LooseVersion(version)
+        elif cmp_op == '=':
+            return LooseVersion(actver) == LooseVersion(version)
+        elif cmp_op == '<':
+            return LooseVersion(actver) < LooseVersion(version)
+        else:
+            return False
+    except TypeError:
+        return True
+
+
+def _is_mod_installed(module_name, version=None):
     """Return True if module *module_name* is installed
     
     If version is not None, checking module version 
     (module must have an attribute named '__version__')
     
-    version may starts with =, >= or > to specify the exact requirement"""
+    version may starts with =, >=, > or < to specify the exact requirement ;
+    multiple conditions may be separated by ';' (e.g. '>=0.13;<1.0')"""
     try:
         mod = __import__(module_name)
         if version is None:
             return True
         else:
+            if ';' in version:
+                output = True
+                for ver in version.split(';'):
+                    output = output and _is_mod_installed(module_name, ver)
+                return output
             match = re.search('[0-9]', version)
             assert match is not None, "Invalid version number"
             symb = version[:match.start()]
             if not symb:
                 symb = '='
-            assert symb in ('>=', '>', '='),\
+            assert symb in ('>=', '>', '=', '<'),\
                    "Invalid version condition '%s'" % symb
             version = version[match.start():]
-            try:
-                actver = getattr(mod, '__version__',
-                                 getattr(mod, 'VERSION', None))
-            except AttributeError:
+            actver = getattr(mod, '__version__', getattr(mod, 'VERSION', None))
+            if actver is None:
                 return False
-            def getvlist(version):
-                """Return an integer list from a version string"""
-                vl = version.split('.')
-                while vl and not vl[-1].isdigit():
-                    vl = vl[:-1]
-                return [int(nb) for nb in vl]
-            vlist = getvlist(version)
-            actvlist = getvlist(actver)
-            actvlist = actvlist[:len(vlist)]
-            vlist = vlist[:len(actvlist)]
-            if not vlist or not actvlist:
-                return False
-            for index, (nb, actnb) in enumerate(zip(vlist, actvlist)):
-                if nb == actnb:
-                    if index == len(vlist)-1 and '=' not in symb:
-                        return False
-                    else:
-                        continue
-                elif actnb < nb:
-                    return False
-                elif actnb > nb and symb == '=':
-                    return False
             else:
-                return True
+                return check_version(actver, version, symb)
     except ImportError:
         return False
 
 
+def is_module_installed(module_name, version=None, interpreter=''):
+    """
+    Check if a module is installed with a given version in a determined
+    interpreter
+    """
+    if interpreter:
+        if not osp.isdir(TEMPDIR):
+            os.mkdir(TEMPDIR)
+        
+        if osp.isfile(interpreter) and ('python' in interpreter):
+            checkver = inspect.getsource(check_version)
+            ismod_inst = inspect.getsource(_is_mod_installed)
+            fd, script = tempfile.mkstemp(suffix='.py', dir=TEMPDIR)
+            with os.fdopen(fd, 'w') as f:
+                f.write("# -*- coding: utf-8 -*-" + "\n\n")
+                f.write("from distutils.version import LooseVersion" + "\n")
+                f.write("import re" + "\n\n")
+                f.write(checkver + "\n")
+                f.write(ismod_inst + "\n")
+                if version:
+                    f.write("print _is_mod_installed('%s','%s')" % (module_name,
+                                                                    version))
+                else:
+                    f.write("print _is_mod_installed('%s')" % module_name)
+            try:
+                output = subprocess.Popen([interpreter, script],
+                                      stdout=subprocess.PIPE).communicate()[0]
+            except subprocess.CalledProcessError:
+                output = 'True'
+            if output:  # TODO: Check why output could be empty!
+                return eval(output)
+            else:
+                return False
+        else:
+            # Try to not take a wrong decision if there is no interpreter
+            # available (needed for the change_pystartup method of ExtConsole
+            # config page)
+            return True
+    else:
+        return _is_mod_installed(module_name, version)
+
+
 if __name__ == '__main__':
     print find_program('hg')
     print shell_split('-q -o -a')
     print shell_split(u'-q "d:\\Python de xxxx\\t.txt" -o -a')
+    print is_module_installed('IPython', '>=0.12')
+    print is_module_installed('IPython', '>=0.13;<1.0')
