@@ -6,7 +6,12 @@
 #      thread. We must find another mechanism to avoid refreshing systematically
 #      remote views for all consoles...!
 
-import os, threading, socket, thread, struct, cPickle as pickle
+import os
+import threading
+import socket
+import thread
+import struct
+import cPickle as pickle
 
 # Local imports
 from spyderlib.utils.misc import fix_reference_name
@@ -15,7 +20,7 @@ from spyderlib.utils.dochelpers import (getargtxt, getdoc, getsource,
                                         getobjdir, isdefined)
 from spyderlib.utils.bsdsocket import (communicate, read_packet, write_packet,
                                        PACKET_NOT_RECEIVED)
-from spyderlib.utils.module_completion import moduleCompletion
+from spyderlib.utils.module_completion import module_completion
 from spyderlib.baseconfig import get_conf_path, get_supported_types, DEBUG
 
 SUPPORTED_TYPES = get_supported_types()
@@ -112,9 +117,10 @@ class Monitor(threading.Thread):
     def __init__(self, host, introspection_port, notification_port,
                  shell_id, timeout, auto_refresh):
         threading.Thread.__init__(self)
-        self.ipython_shell = None
-        self.ipython_kernel = None
         self.setDaemon(True)
+
+        self.ipykernel = None
+        self.ipython_shell = None
         
         self.pdb_obj = None
         
@@ -127,6 +133,9 @@ class Monitor(threading.Thread):
         self.inputhook_flag = False
         self.first_inputhook_call = True
         
+        # To grab the IPython internal namespace
+        self.ip = None
+        
         # Connecting to introspection server
         self.i_request = socket.socket( socket.AF_INET )
         self.i_request.connect( (host, introspection_port) )
@@ -143,7 +152,6 @@ class Monitor(threading.Thread):
                        "is_array": self.is_array,
                        "is_image": self.is_image,
                        "get_globals_keys": self.get_globals_keys,
-                       "getcomplist": self.getcomplist,
                        "getmodcomplist": self.getmodcomplist,
                        "getcdlistdir": _getcdlistdir,
                        "getcwd": self.getcwd,
@@ -158,6 +166,7 @@ class Monitor(threading.Thread):
                        "set_monitor_auto_refresh": self.set_auto_refresh,
                        "set_remote_view_settings":
                                                 self.set_remote_view_settings,
+                       "set_spyder_breakpoints": self.set_spyder_breakpoints,
                        "__get_dir__": self.get_dir,
                        "__iscallable__": self.iscallable,
                        "__get_arglist__": self.get_arglist,
@@ -205,19 +214,16 @@ class Monitor(threading.Thread):
                 self._mglobals = glbs
             else:
                 glbs = self._mglobals
-            if self.ipython_kernel is None and '__ipythonkernel__' in glbs:
-                self.ipython_kernel = glbs['__ipythonkernel__']
+            if self.ipykernel is None and '__ipythonkernel__' in glbs:
+                self.ipykernel = glbs['__ipythonkernel__']
                 communicate(self.n_request,
-                            dict(command="ipython_kernel",
-                                 data=self.ipython_kernel.connection_file))
+                            dict(command="ipykernel",
+                                 data=self.ipykernel.connection_file))
             if self.ipython_shell is None and '__ipythonshell__' in glbs:
-                # IPython >=v0.11
+                # IPython 0.13+ kernel
                 self.ipython_shell = glbs['__ipythonshell__']
-                if not hasattr(self.ipython_shell, 'user_ns'):
-                    # IPython v0.10
-                    self.ipython_shell = self.ipython_shell.IP
-                self.ipython_shell.modcompletion = moduleCompletion
                 glbs = self.ipython_shell.user_ns
+                self.ip = self.ipython_shell.get_ipython()
             self._mglobals = glbs
             return glbs
     
@@ -225,14 +231,24 @@ class Monitor(threading.Thread):
         """Return current namespace, i.e. globals() if not debugging,
         or a dictionary containing both locals() and globals() 
         for current frame when debugging"""
+        ns = {}
         glbs = self.mglobals()
+
         if self.pdb_frame is None:
-            return glbs
+            ns.update(glbs)
         else:
-            ns = {}
             ns.update(glbs)
             ns.update(self.pdb_locals)
-            return ns
+        
+        # Add magics to ns so we can show help about them on the Object
+        # Inspector
+        if self.ip:
+            line_magics = self.ip.magics_manager.magics['line']
+            cell_magics = self.ip.magics_manager.magics['cell']
+            ns.update(line_magics)
+            ns.update(cell_magics)
+        
+        return ns
     
     def get_reference_namespace(self, name):
         """Return namespace where reference name is defined,
@@ -298,6 +314,12 @@ class Monitor(threading.Thread):
         communicate(self.n_request,
                     dict(command="pdb_step", data=(fname, lineno)))
 
+    def set_spyder_breakpoints(self):
+        """Set all Spyder breakpoints in active pdb session"""
+        if not self.pdb_obj:
+            return
+        self.pdb_obj.set_spyder_breakpoints()    
+    
     def notify_open_file(self, fname, lineno=1):
         """Open file in Spyder's editor"""
         communicate(self.n_request,
@@ -345,32 +367,17 @@ class Monitor(threading.Thread):
         """Get object documentation"""
         obj, valid = self._eval(objtxt)
         if valid:
-            return unicode(getdoc(obj))
+            return getdoc(obj)
     
     def get_source(self, objtxt):
         """Get object source"""
         obj, valid = self._eval(objtxt)
         if valid:
             return getsource(obj)
-    
-    def getcomplist(self, name):
-        """Return completion list for object named *name*
-        ** IPython only **"""
-        if self.ipython_shell:
-            complist = self.ipython_shell.complete(name)
-            if len(complist) == 2 and isinstance(complist[1], list):
-                # IPython v0.11
-                return complist[1]
-            else:
-                # IPython v0.10
-                return complist
             
-    def getmodcomplist(self, name):
+    def getmodcomplist(self, name, path):
         """Return module completion list for object named *name*"""
-        if self.ipython_shell:
-            return self.ipython_shell.modcompletion(name)
-        else:
-            return moduleCompletion(name)
+        return module_completion(name, path)
                 
     #------ Other
     def is_array(self, name):
@@ -393,17 +400,15 @@ class Monitor(threading.Thread):
 
     def getcwd(self):
         """Return current working directory"""
-        if self.ipython_shell:
-            return self.ipython_shell.magic_pwd()
-        else:
-            return os.getcwdu()
+        return os.getcwdu()
     
     def setcwd(self, dirname):
         """Set current working directory"""
-        if self.ipython_shell:
-            self.ipython_shell.magic_cd("-q "+dirname)
-        else:
-            return os.chdir(dirname)
+        try:
+            dirname = dirname.decode('utf-8')
+        except (UnicodeError, TypeError):
+            pass
+        return os.chdir(dirname)
             
     def getenv(self):
         """Return os.environ"""
@@ -440,8 +445,8 @@ class Monitor(threading.Thread):
         """
         settings = self.remote_view_settings
         if settings:
-            more_excluded_names = ['In', 'Out'] if self.ipython_shell else None
             ns = self.get_current_namespace()
+            more_excluded_names = ['In', 'Out'] if self.ipython_shell else None
             remote_view = make_remote_view(ns, settings, more_excluded_names)
             communicate(self.n_request,
                         dict(command="remote_view", data=remote_view))
