@@ -17,14 +17,13 @@ locale_codec = QTextCodec.codecForLocale()
 from spyderlib.qt.compat import getexistingdirectory
 
 # Local imports
-from spyderlib.utils.qthelpers import (create_toolbutton, create_action,
-                                       get_std_icon, DialogManager,
-                                       add_actions)
+from spyderlib.utils.qthelpers import (get_icon, get_std_icon, add_actions,
+                                       create_toolbutton, create_action,
+                                       DialogManager)
 from spyderlib.utils.environ import RemoteEnvDialog
 from spyderlib.utils.programs import get_python_args
 from spyderlib.utils.misc import get_python_executable
-from spyderlib.baseconfig import _, get_module_source_path
-from spyderlib.config import get_icon
+from spyderlib.baseconfig import _, get_module_source_path, DEBUG
 from spyderlib.widgets.shell import PythonShellWidget
 from spyderlib.widgets.externalshell.namespacebrowser import NamespaceBrowser
 from spyderlib.widgets.externalshell.monitor import communicate, write_packet
@@ -34,9 +33,9 @@ from spyderlib.widgets.dicteditor import DictEditor
 
 
 class ExtPythonShellWidget(PythonShellWidget):
-    def __init__(self, parent, history_filename, debug=False, profile=False):
-        PythonShellWidget.__init__(self, parent, history_filename,
-                                   debug, profile)
+    def __init__(self, parent, history_filename, profile=False):
+        PythonShellWidget.__init__(self, parent, history_filename, profile)
+        self.path = []
     
     def set_externalshell(self, externalshell):
         # ExternalShellBase instance:
@@ -119,14 +118,11 @@ class ExtPythonShellWidget(PythonShellWidget):
         """Return True if object is defined"""
         return self.ask_monitor("isdefined('%s', force_import=%s)"
                                 % (objtxt, force_import))
-
-    def get_completion(self, objtxt):
-        """Return completion list associated to object name"""
-        return self.ask_monitor("getcomplist('%s')" % objtxt)
         
     def get_module_completion(self, objtxt):
         """Return module completion list associated to object name"""
-        return self.ask_monitor("getmodcomplist('%s')" % objtxt)
+        return self.ask_monitor("getmodcomplist('%s', %s)" % \
+                                                           (objtxt, self.path))
     
     def get_cwd(self):
         """Return shell current working directory"""
@@ -148,14 +144,17 @@ class ExtPythonShellWidget(PythonShellWidget):
         """Return sys.path[:]"""
         return self.ask_monitor("getsyspath()")
 
+    def set_spyder_breakpoints(self):
+        """Set Spyder breakpoints into debugging session"""
+        return self.ask_monitor("set_spyder_breakpoints()")
+        
 
 class ExternalPythonShell(ExternalShellBase):
     """External Shell widget: execute Python script in a separate process"""
     SHELL_CLASS = ExtPythonShellWidget
     def __init__(self, parent=None, fname=None, wdir=None,
                  interact=False, debug=False, path=[], python_args='',
-                 ipython_shell=False, ipython_kernel=False,
-                 arguments='', stand_alone=None,
+                 ipykernel=False, arguments='', stand_alone=None,
                  umd_enabled=True, umd_namelist=[], umd_verbose=True,
                  pythonstartup=None, pythonexecutable=None,
                  monitor_enabled=True, mpl_patch_enabled=True,
@@ -214,14 +213,14 @@ class ExternalPythonShell(ExternalShellBase):
         assert isinstance(arguments, basestring)
         self.arguments = arguments
         
-        self.is_ipython_shell = ipython_shell
-        self.is_ipython_kernel = ipython_kernel
-        if self.is_ipython_shell or self.is_ipython_kernel:
+        self.connection_file = None
+        self.is_ipykernel = ipykernel
+        if self.is_ipykernel:
             interact = False
-            # Running our custom startup script for IPython sessions:
-            # (see spyderlib/widgets/externalshell/startup.py)
+            # Running our custom startup script for IPython kernels:
+            # (see spyderlib/widgets/externalshell/start_ipython_kernel.py)
             self.fname = get_module_source_path(
-                            'spyderlib.widgets.externalshell', 'startup.py')
+                'spyderlib.widgets.externalshell', 'start_ipython_kernel.py')
         
         self.shell.set_externalshell(self)
 
@@ -237,6 +236,7 @@ class ExternalPythonShell(ExternalShellBase):
         
         # Additional python path list
         self.path = path
+        self.shell.path = path
         
     def set_introspection_socket(self, introspection_socket):
         self.introspection_socket = introspection_socket
@@ -307,7 +307,7 @@ The process may not exit as a result of clicking this button
         return actions
 
     def is_interpreter(self):
-        """Return True if shellwidget is a Python/IPython interpreter"""
+        """Return True if shellwidget is a Python interpreter"""
         return self.is_interpreter
         
     def get_shell_widget(self):
@@ -340,9 +340,7 @@ The process may not exit as a result of clicking this button
         ExternalShellBase.set_buttons_runnning_state(self, state)
         self.interact_action.setEnabled(not state and not self.is_interpreter)
         self.debug_action.setEnabled(not state and not self.is_interpreter)
-        self.args_action.setEnabled(not state and
-                                    (not self.is_interpreter or\
-                                     self.is_ipython_shell))
+        self.args_action.setEnabled(not state and not self.is_interpreter)
         if state:
             if self.arguments:
                 argstr = _("Arguments: %s") % self.arguments
@@ -380,7 +378,7 @@ The process may not exit as a result of clicking this button
         self.shell.clear()
             
         self.process = QProcess(self)
-        if self.merge_output_channels or self.is_ipython_shell:
+        if self.merge_output_channels:
             self.process.setProcessChannelMode(QProcess.MergedChannels)
         else:
             self.process.setProcessChannelMode(QProcess.SeparateChannels)
@@ -393,12 +391,16 @@ The process may not exit as a result of clicking this button
 
         #-------------------------Python specific-------------------------------
         # Python arguments
-        p_args = ['-u'] + get_python_args(self.fname, self.python_args,
-                                          self.interact_action.isChecked(),
-                                          self.debug_action.isChecked(),
-                                          self.arguments)
+        p_args = ['-u']
+        if DEBUG:
+            p_args += ['-v']
+        p_args += get_python_args(self.fname, self.python_args,
+                                  self.interact_action.isChecked(),
+                                  self.debug_action.isChecked(),
+                                  self.arguments)
         
         env = [unicode(_path) for _path in self.process.systemEnvironment()]
+
         if self.pythonstartup:
             env.append('PYTHONSTARTUP=%s' % self.pythonstartup)
         
@@ -418,7 +420,7 @@ The process may not exit as a result of clicking this button
             self.connect(self.notification_thread,
                          SIGNAL('new_ipython_kernel(QString)'),
                          lambda args:
-                         self.emit(SIGNAL('create_ipython_frontend(QString)'),
+                         self.emit(SIGNAL('create_ipython_client(QString)'),
                          args))
             self.connect(self.notification_thread,
                          SIGNAL('open_file(QString,int)'),
@@ -454,14 +456,9 @@ The process may not exit as a result of clicking this button
             env.append('UMD_ENABLED=%r' % self.umd_enabled)
             env.append('UMD_NAMELIST=%s' % ','.join(self.umd_namelist))
             env.append('UMD_VERBOSE=%r' % self.umd_verbose)
-        
-        # IPython related configuration
-        if self.is_ipython_shell:
-            env.append('IPYTHON=True')
-            # Do not call msvcrt.getch in IPython.genutils.page_more:
-            env.append('TERM=emacs')
-        elif self.is_ipython_kernel:
-            env.append('IPYTHON_KERNEL=True')
+
+        # IPython kernel
+        env.append('IPYTHON_KERNEL=%r' % self.is_ipykernel)
             
         pathlist = []
 
@@ -474,8 +471,7 @@ The process may not exit as a result of clicking this button
         
         # Adding path list to PYTHONPATH environment variable
         add_pathlist_to_PYTHONPATH(env, pathlist)
-        
-        self.process.setEnvironment(env)
+
         #-------------------------Python specific-------------------------------
                         
         self.connect(self.process, SIGNAL("readyReadStandardOutput()"),
@@ -496,6 +492,26 @@ The process may not exit as a result of clicking this button
         executable = self.pythonexecutable
         if executable is None:
             executable = get_python_executable()
+        
+        # Fixes for our Mac app:
+        # 1. PYTHONPATH and PYTHONHOME are set while bootstrapping the app,
+        #    but their values are messing sys.path for external interpreters
+        #    (e.g. EPD) so we need to remove them from the environment.
+        # 2. Add this file's dir to PYTHONPATH. This will make every external
+        #    interpreter to use our sitecustomize script.
+        # 3. Remove PYTHONOPTIMIZE from env so that we can have assert
+        #    statements working with our interpreters (See Issue 1281)
+        if sys.platform == 'darwin' and 'Spyder.app' in __file__:
+            env.append('SPYDER_INTERPRETER=%s' % executable)
+            if 'Spyder.app' not in executable:
+                env = [p for p in env if not (p.startswith('PYTHONPATH') or \
+                                              p.startswith('PYTHONHOME'))] # 1.
+
+                env.append('PYTHONPATH=%s' % osp.dirname(__file__))        # 2.
+
+            env = [p for p in env if not p.startswith('PYTHONOPTIMIZE')]   # 3.
+
+        self.process.setEnvironment(env)
         self.process.start(executable, p_args)
         #-------------------------Python specific-------------------------------
             
@@ -509,6 +525,12 @@ The process may not exit as a result of clicking this button
             self.emit(SIGNAL('started()'))
             
         return self.process
+
+    def finished(self, exit_code, exit_status):
+        """Reimplement ExternalShellBase method"""
+        ExternalShellBase.finished(self, exit_code, exit_status)
+        self.introspection_socket = None
+
     
 #===============================================================================
 #    Input/Output
@@ -524,20 +546,18 @@ The process may not exit as a result of clicking this button
         QApplication.processEvents()
         
     def send_to_process(self, text):
+        if not self.is_running():
+            return
+            
         if not isinstance(text, basestring):
             text = unicode(text)
-        if self.install_qt_inputhook and not self.is_ipython_shell:
-            # For now, the Spyder's input hook does not work with IPython:
-            # with IPython v0.10 or non-Windows platforms, this is not a
-            # problem. However, with IPython v0.11 on Windows, this will be
-            # fixed by patching IPython to force it to use our inputhook.
-            if self.introspection_socket is not None:
-                communicate(self.introspection_socket,
-                            "toggle_inputhook_flag(True)")
+        if self.install_qt_inputhook and self.introspection_socket is not None:
+            communicate(self.introspection_socket,
+                        "toggle_inputhook_flag(True)")
 #            # Socket-based alternative (see input hook in sitecustomize.py):
 #            while self.local_server.hasPendingConnections():
 #                self.local_server.nextPendingConnection().write('go!')
-        if not self.is_ipython_shell and text.startswith(('%', '!')):
+        if text.startswith(('%', '!')):
             text = 'evalsc(r"%s")\n' % text
         if not text.endswith('\n'):
             text += '\n'
