@@ -8,6 +8,8 @@
 
 # pylint: disable=C0103
 
+from __future__ import with_statement
+
 from spyderlib.qt.QtGui import (QVBoxLayout, QLabel, QHBoxLayout, QWidget,
                                 QFileIconProvider, QMessageBox, QInputDialog,
                                 QLineEdit, QPushButton)
@@ -23,9 +25,8 @@ import xml.etree.ElementTree as ElementTree
 
 # Local imports
 from spyderlib.utils import misc
-from spyderlib.utils.qthelpers import get_std_icon, create_action
-from spyderlib.baseconfig import _, STDERR
-from spyderlib.config import get_icon, get_image_path
+from spyderlib.utils.qthelpers import get_icon, get_std_icon, create_action
+from spyderlib.baseconfig import _, STDERR, get_image_path
 from spyderlib.widgets.explorer import FilteredDirView, listdir, fixpath
 from spyderlib.widgets.formlayout import fedit
 from spyderlib.widgets.pathmanager import PathManager
@@ -116,11 +117,19 @@ class Project(object):
         
     def load(self):
         """Load project data"""
+        fname = self.__get_project_config_path()
         try:
-            data = cPickle.loads(file(self.__get_project_config_path(),
-                                      'U').read())
-        except (IOError, OSError):
-            self.ioerror_flag = True
+            # Old format (Spyder 2.0-2.1 for Python 2)
+            with open(fname, 'U') as fdesc:
+                data = cPickle.loads(fdesc.read())
+        except cPickle.PickleError:
+            try:
+                # New format (Spyder >=2.2 for Python 2 and Python 3)
+                with open(fname, 'rb') as fdesc:
+                    data = cPickle.loads(fdesc.read())
+            except (IOError, OSError, cPickle.PickleError):
+                self.ioerror_flag = True
+                return
         # Compatibilty with old project explorer file format:
         if 'relative_pythonpath' not in data:
             print >>STDERR, "Warning: converting old configuration file " \
@@ -137,7 +146,8 @@ class Project(object):
         for attr in self.CONFIG_ATTR:
             data[attr] = getattr(self, attr)
         try:
-            cPickle.dump(data, file(self.__get_project_config_path(), 'w'))
+            with open(self.__get_project_config_path(), 'wb') as fdesc:
+                cPickle.dump(data, fdesc, 2)
         except (IOError, OSError):
             self.ioerror_flag = True
         
@@ -278,23 +288,32 @@ class Workspace(object):
         return osp.join(self.root_path, self.CONFIG_NAME)
         
     def load(self):
-        """Load project data"""
-        fdesc = file(self.__get_workspace_config_path(), 'U')
+        """Load workspace data"""
+        fname = self.__get_workspace_config_path()
         try:
-            data = cPickle.loads(fdesc.read())
-        except (IOError, OSError):
-            self.ioerror_flag = True
+            # Old format (Spyder 2.0-2.1 for Python 2)
+            with open(fname, 'U') as fdesc:
+                data = cPickle.loads(fdesc.read())
+        except cPickle.PickleError:
+            try:
+                # New format (Spyder >=2.2 for Python 2 and Python 3)
+                with open(fname, 'rb') as fdesc:
+                    data = cPickle.loads(fdesc.read())
+            except (IOError, OSError, cPickle.PickleError):
+                self.ioerror_flag = True
+                return
         for attr in self.CONFIG_ATTR:
             setattr(self, attr, data[attr])
         self.save()
     
     def save(self):
-        """Save project data"""
+        """Save workspace data"""
         data = {}
         for attr in self.CONFIG_ATTR:
             data[attr] = getattr(self, attr)
         try:
-            cPickle.dump(data, file(self.__get_workspace_config_path(), 'w'))
+            with open(self.__get_workspace_config_path(), 'wb') as fdesc:
+                cPickle.dump(data, fdesc, 2)
         except (IOError, OSError):
             self.ioerror_flag = True
         
@@ -321,6 +340,11 @@ class Workspace(object):
     def is_file_in_workspace(self, fname):
         """Return True if file *fname* is in one of the projects"""
         return any([proj.is_file_in_project(fname) for proj in self.projects])
+        
+    def is_file_in_closed_project(self, fname):
+        """Return True if file *fname* is in one of the closed projects"""
+        return any([proj.is_file_in_project(fname) for proj in self.projects
+                    if not proj.is_opened()])
     
     def is_in_pythonpath(self, dirname):
         """Return True if dirname is in workspace's PYTHONPATH"""
@@ -715,6 +739,9 @@ class ExplorerTreeWidget(FilteredDirView):
         project = self.workspace.add_project(folder)
         self.set_folder_names(self.workspace.get_folder_names())
         self.parent_widget.emit(SIGNAL("pythonpath_changed()"))
+        
+        self.check_for_io_errors()
+        
         return project
         
     def open_projects(self, projects, open_related=True):
@@ -729,6 +756,7 @@ class ExplorerTreeWidget(FilteredDirView):
         """Close projects"""
         self.workspace.close_projects(projects)
         self.parent_widget.emit(SIGNAL("pythonpath_changed()"))
+        self.parent_widget.emit(SIGNAL("projects_were_closed()"))
         self.reset_icon_provider()
         for project in projects:
             index = self.get_index(project.root_path)
